@@ -1,2 +1,2 @@
 from tinytim.filter import column_filter, sample, only_columns, indexes_from_filter, filter_list_by_indexes, filter_by_indexes
-from tinytim.filter import sample_indexes
\ No newline at end of file
+from tinytim.filter import sample_indexes, filter_by_columns
\ No newline at end of file
