@@ -1,10 +1,11 @@
 from __future__ import annotations
 import copy
-from typing import Iterable, List, Mapping, MutableMapping, Optional, Sequence, Union, Iterator
-from typing import Any, Callable, MutableSequence, Generator
+from typing import Dict, List, Mapping, Optional, Sequence, Union, Iterator
+from typing import Any, Callable, Generator
 from enum import Enum
 
 from tabulate import tabulate
+from hasattrs import has_mapping_attrs
 
 from tinytable.column import Column
 from tinytable.row import Row
@@ -27,6 +28,9 @@ import tinytable.functional.group as group
 import tinytable.functional.join as join
 import tinytable.functional.na as na
 
+from tinytable.types import DataDict, data_dict, DataMapping
+ColumnNames = Sequence[str]
+
 
 class JoinStrategy(str, Enum):
     left = 'left'
@@ -35,31 +39,51 @@ class JoinStrategy(str, Enum):
     full = 'full'
 
 
-class Table(MutableMapping):
+class Table:
     """Data table organized into {column_name: list[values]}
     
        A pure Python version of Pandas DataFrame.
     """
-    def __init__(self, data: Union[MutableMapping, Sequence[Sequence]] = {}, labels=None, columns=None) -> None:
-        if columns is not None and not isinstance(data, Mapping):
-            data = {col: values for col, values in zip(columns, data)}
-        if columns is not None and isinstance(data, Mapping):
-            data = {col: values for col, values in zip(columns, data.values())}
-        if not isinstance(data, Mapping):
-            data = {str(i): values for i, values in zip(range(len(data)), data)}
-
-        self.data = data
-        self._store_data()
+    def __init__(
+        self,
+        data: Union[DataMapping, Sequence[Sequence]] = {},
+        labels: Optional[Sequence] = None,
+        columns: Optional[ColumnNames] = None
+    ) -> None:
+        """
+        data passed can be Mapping[str, Iterable] ({column_name: column_values}) or
+        Sequence[Sequence] (sequence of row values) with column names passed in columns parameter.
+        """
+        data = self._transform_data_input(data, columns)
+        self.data: DataDict = data
         self._validate()
-        self.labels = labels if labels is None else list(labels)
+        self.labels: Union[None, list] = labels if labels is None else list(labels)
 
-    def _store_data(self):
-        for col in self.data:
-            self._store_column(col, self.data[col])
+    def _transform_data_input(
+        self,
+        data: Union[DataMapping, Sequence[Sequence]],
+        columns: Optional[ColumnNames] = None
+    ) -> Dict[str, list]:
+        out: Dict[str, list]
+        if columns is not None and not has_mapping_attrs(data):
+            # data is a sequence of sequences and column names passed
+            return rows.row_values_to_data(list(data), columns)
+        return data_dict(data)
 
-    def _store_column(self, column_name: str, values: Iterable) -> Union[None, Table]:
-        values = list(values)
-        self.data[column_name] = values
+    @classmethod
+    def from_records(
+        cls,
+        data: Sequence[Sequence],
+        columns: Sequence[str],
+        labels: Optional[Sequence] = None
+    ) -> Table:
+        """Convert sequence of row values to Table"""
+        return Table(rows.row_values_to_data(data, columns), labels=labels)
+
+    @classmethod
+    def from_dict(cls, data: DataDict, columns=None) -> Table:
+        """Costruct Table from dict of column values"""
+        return Table(data, columns)
         
     def __len__(self) -> int:
         return features.row_count(self.data)
@@ -117,7 +141,7 @@ class Table(MutableMapping):
                 return self[index]
         raise TypeError('key must be str for column selection, int for row selection or slice for subset of Table rows.')
     
-    def __setitem__(self, key: Union[str, int], values: MutableSequence) -> None:
+    def __setitem__(self, key: Union[str, int], values: list) -> None:
         if type(key) == str:
             column_name: str = str(key)
             self.edit_column(column_name, values)
@@ -147,7 +171,7 @@ class Table(MutableMapping):
         return features.column_names(self.data)
 
     @columns.setter
-    def columns(self, values: MutableSequence) -> None:
+    def columns(self, values: List[str]) -> None:
         """Set the value of the column names."""
         self.replace_column_names(values)
 
@@ -168,10 +192,10 @@ class Table(MutableMapping):
         indexes = filter.indexes_from_filter(f)
         return self.filter_by_indexes(indexes)
 
-    def only_columns(self, column_names: MutableSequence[str]) -> Table:
+    def only_columns(self, column_names: List[str]) -> Table:
         """Return new Table with only column_names Columns."""
         d = filter.only_columns(self.data, column_names)
-        return Table(d, self.labels)
+        return Table(data_dict(d), self.labels)
 
     def _convert_index(self, index: int) -> int:
         if index < 0:
@@ -234,20 +258,20 @@ class Table(MutableMapping):
     def itertuples(self) -> Generator[tuple, None, None]:
         return rows.itertuples(self.data)
     
-    def edit_row(self, index: int, values: Union[Mapping, MutableSequence], inplace=True) -> Union[None, Table]:
+    def edit_row(self, index: int, values: Union[Mapping, Sequence], inplace=True) -> Union[None, Table]:
         if inplace:
             if isinstance(values, Mapping):
                 edit.edit_row_items_inplace(self.data, index, values)
-            elif isinstance(values, MutableSequence):
+            elif isinstance(values, Sequence):
                 edit.edit_row_values_inplace(self.data, index, values)
         else:
             if isinstance(values, Mapping):
                 data = edit.edit_row_items(self.data, index, values)
-            elif isinstance(values, MutableSequence):
+            elif isinstance(values, Sequence):
                 data = edit.edit_row_values(self.data, index, values)
             return Table(data, copy.copy(self.labels))
             
-    def edit_column(self, column_name: str, values: MutableSequence, inplace=True) ->Union[None, Table]:
+    def edit_column(self, column_name: str, values: Sequence, inplace=True) ->Union[None, Table]:
         if inplace:
             edit.edit_column_inplace(self.data, column_name, values)
             return 
@@ -263,12 +287,12 @@ class Table(MutableMapping):
     def copy(self, deep=False) -> Table:
         if deep:
              return Table(data_copy.deepcopy_table(self.data), copy.deepcopy(self.labels))
-        return Table(data_copy.copy_table(self.data), copy.copy(self.labels))
+        return Table(data_dict(data_copy.copy_table(self.data)), copy.copy(self.labels))
 
     def cast_column_as(self, column_name: str, data_type: Callable) -> None:
         self.data[column_name] = [data_type(value) for value in self.data[column_name]]
 
-    def replace_column_names(self, new_keys: MutableSequence[str]) -> None:
+    def replace_column_names(self, new_keys: Sequence[str]) -> None:
         if len(new_keys) != len(self.data.keys()):
             raise ValueError('new_keys must be same len as dict keys.')
         for new_key, old_key in zip(new_keys, self.data.keys()):
@@ -319,10 +343,10 @@ class Table(MutableMapping):
         return None if self.labels is None else self.labels[5:]
 
     def head(self, n: int = 5) -> Table:
-        return Table(features.head(self.data, n), self.label_head(n))
+        return Table(data_dict(features.head(self.data, n)), self.label_head(n))
 
     def tail(self, n: int = 5) -> Table:
-        return Table(features.tail(self.data, n), self.label_tail(n))
+        return Table(data_dict(features.tail(self.data, n)), self.label_tail(n))
 
     def nunique(self) -> dict[str, int]:
         """Count number of distinct values in each column.
@@ -330,39 +354,61 @@ class Table(MutableMapping):
         """
         return utils.nunique(self.data)
 
-    def filter_by_indexes(self, indexes: MutableSequence[int]) -> Table:
+    def filter_by_indexes(self, indexes: Sequence[int]) -> Table:
         """return only rows in indexes"""
         labels = None if self.labels is None else filter.filter_list_by_indexes(self.labels, indexes)
-        return Table(filter.filter_by_indexes(self.data, indexes), labels=labels)
+        data = data_dict(filter.filter_by_indexes(self.data, indexes))
+        if len(data) == 0:
+            self.labels = None
+        return Table(data, labels=labels)
+
+    def filter_by_indexes_inplace(self, indexes: Sequence[int]) -> None:
+        """return only rows in indexes"""
+        labels = None if self.labels is None else filter.filter_list_by_indexes(self.labels, indexes)
+        data = data_dict(filter.filter_by_indexes(self.data, indexes))
+        if len(data) == 0:
+            self.labels = None
+        self.labels = labels
+        self.data = data
+    
+    def filter_by_columns(self, columns: Sequence[str]) -> Table:
+        data = data_dict(filter.filter_by_columns(self.data, columns))
+        return Table(data, labels=self.labels)
+
+    def filter_by_columns_inplace(self, columns: Sequence[str]) -> None:
+        data = data_dict(filter.filter_by_columns(self.data, columns))
+        self.data = data
+        if len(self.data) == 0:
+            self.labels = None
 
     def sample(self, n, random_state=None) -> Table:
         """return random sample of rows"""
         indexes = filter.sample_indexes(self.data, n, random_state)
         labels = None if self.labels is None else filter.filter_list_by_indexes(self.labels, indexes)
-        return Table(filter.filter_by_indexes(self.data, indexes), labels=labels)
+        return Table(data_dict(filter.filter_by_indexes(self.data, indexes)), labels=labels)
 
     def groupby(self, by: Union[str, Sequence]) -> Group:
         return Group([(value, Table(data)) for value, data in group.groupby(self.data, by)], by)
 
-    def inner_join(self, other: Mapping[str, Sequence], left_on, right_on=None) -> Table:
+    def inner_join(self, other: DataMapping, left_on, right_on=None) -> Table:
         data = join.inner_join(self.data, other, left_on, right_on)
         return Table(data)
 
-    def left_join(self, other: Mapping[str, Sequence], left_on, right_on=None) -> Table:
+    def left_join(self, other: DataMapping, left_on, right_on=None) -> Table:
         data = join.left_join(self.data, other, left_on, right_on)
         return Table(data)
 
-    def right_join(self, other: Mapping[str, Sequence], left_on, right_on=None) -> Table:
+    def right_join(self, other: DataMapping, left_on, right_on=None) -> Table:
         data = join.right_join(self.data, other, left_on, right_on)
         return Table(data)
 
-    def full_join(self, other: Mapping[str, Sequence], left_on, right_on=None) -> Table:
+    def full_join(self, other: DataMapping, left_on, right_on=None) -> Table:
         data = join.full_join(self.data, other, left_on, right_on)
         return Table(data)
 
     def join(
         self,
-        other: Mapping[str, Sequence],
+        other: DataMapping,
         left_on,
         right_on=None,
         how: JoinStrategy = JoinStrategy.left
@@ -439,15 +485,15 @@ class Table(MutableMapping):
         """
         data = na.fillna(self.data, value, method, axis, inplace, limit, na_value)
         if data is not None:
-            return Table(data, self.labels)
+            return Table(data_dict(data), self.labels)
                 
     def isna(self, na_value=None) -> Table:
         data = na.isna(self.data, na_value)
-        return Table(data, self.labels)
+        return Table(data_dict(data), self.labels)
 
     def notna(self, na_value=None) -> Table:
         data = na.notna(self.data, na_value)
-        return Table(data, self.labels)
+        return Table(data_dict(data), self.labels)
 
     isnull = isna
     notnull = notna
@@ -482,9 +528,30 @@ class Table(MutableMapping):
         Table | None
             Table with missing values removed or None if inplace=True
         """
-        data = na.dropna(self.data, axis, how, thresh, subset, inplace, na_value)
-        if data is not None:
-            return Table(data, self.labels)
+        # Get the remaining indexes/column names after dropping na rows/columns
+        if thresh is not None:
+            remaining = na.dropna_thresh(self.data, thresh, axis, subset, na_value, remaining=True)
+        elif how == 'any':
+            remaining = na.dropna_any(self.data, axis, subset, na_value, remaining=True)
+        elif how == 'all':
+            remaining = na.dropna_all(self.data, axis, subset, na_value, remaining=True)
+        else:
+            raise ValueError('how must be "any" or "all" if thresh is not None')
+        # Filter by indexes or columns
+        if axis in [1, 'columns']:
+            remaining_columns: List[str] = [str(name) for name in remaining]
+            if inplace:
+                self.filter_by_columns_inplace(remaining_columns)
+            else:
+                return self.filter_by_columns(remaining_columns)
+        elif axis in [0, 'rows']:
+            remaining_indexes = [int(index) for index in remaining]
+            if inplace:
+                self.filter_by_indexes_inplace(remaining_indexes)
+            else:
+                return self.filter_by_indexes(remaining_indexes)
+        else:
+            raise ValueError('axis but be 0, 1, "columns", or "rows"')
 
 
 def read_csv(path: str, names: Optional[Sequence[str]] = None):
